require 'spec_helper'

describe OmniAuth::Strategies::OutrightMental do
  let(:access_token) { double('AccessToken', :options => {}) }
  let(:parsed_response) { double('ParsedResponse') }
  let(:response) { double('Response', :parsed => parsed_response) }

  let(:enterprise_site)          { 'https://some.other.site.com/api/v3' }
  let(:enterprise_authorize_url) { 'https://some.other.site.com/auth/authorize' }
  let(:enterprise_token_url)     { 'https://some.other.site.com/auth/token' }
  let(:enterprise) do
    OmniAuth::Strategies::OutrightMental.new('OUTRIGHTMENTAL_KEY', 'OUTRIGHTMENTAL_SECRET',
        {
            :client_options => {
                :site => enterprise_site,
                :authorize_url => enterprise_authorize_url,
                :token_url => enterprise_token_url
            }
        }
    )
  end

  subject do
    OmniAuth::Strategies::OutrightMental
  end

  context "client options" do
    it 'should have correct site' do
      instance = subject.new({})
      instance.options.client_options.site.should eq("https://ont.io/")
    end

    it 'should have correct authorize url' do
      instance = subject.new({})
      instance.options.client_options.authorize_url.should eq('https://ont.io/oauth/authorize')
    end

    it 'should have correct token url' do
      instance = subject.new({})
      instance.options.client_options.token_url.should eq('https://ont.io/oauth/token')
    end

    describe "should be overrideable" do
      it "for site" do
        enterprise.options.client_options.site.should eq(enterprise_site)
      end

      it "for authorize url" do
        enterprise.options.client_options.authorize_url.should eq(enterprise_authorize_url)
      end

      it "for token url" do
        enterprise.options.client_options.token_url.should eq(enterprise_token_url)
      end
    end
  end

  context "#email_access_allowed?" do
    it "should not allow email if scope is nil" do
      instance = subject.new({})
      instance.options['scope'].should be_nil
      instance.should_not be_email_access_allowed
    end

    it "should allow email if scope is user" do
      instance = subject.new({})
      instance.options['scope'] = 'user'
      instance.should be_email_access_allowed
    end

    it "should allow email if scope is a bunch of stuff including user" do
      instance = subject.new({})
      instance.options['scope'] = 'public_repo,user,repo,delete_repo,gist'
      instance.should be_email_access_allowed
    end

    it "should not allow email if scope does not grant email access" do
      instance = subject.new({})
      instance.options['scope'] = 'repo,user:follow'
      instance.should_not be_email_access_allowed
    end

    it "should assume email access not allowed if scope is something currently not documented " do
      instance = subject.new({})
      instance.options['scope'] = 'currently_not_documented'
      instance.should_not be_email_access_allowed
    end
  end

  context "#email" do
    it "should return email from raw_info if available" do
      instance = subject.new({})
      instance.stub(:raw_info).and_return({'email' => 'you@example.com'})
      instance.email.should eq('you@example.com')
    end

    it "should return nil if there is no raw_info and email access is not allowed" do
      instance = subject.new({})
      instance.stub(:raw_info).and_return({})
      instance.email.should be_nil
    end

    it "should return the primary email if there is no raw_info and email access is allowed" do
      instance = subject.new({})
      emails = [
        { 'email' => 'secondary@example.com', 'primary' => false },
        { 'email' => 'primary@example.com',   'primary' => true }
      ]
      instance.stub(:raw_info).and_return({})
      instance.options['scope'] = 'user'
      instance.stub(:emails).and_return(emails)
      instance.email.should eq('primary@example.com')
    end

    it "should return the first email if there is no raw_info and email access is allowed" do
      instance = subject.new({})
      emails = [
        { 'email' => 'first@example.com',   'primary' => false },
        { 'email' => 'second@example.com',  'primary' => false }
      ]
      instance.stub(:raw_info).and_return({})
      instance.options['scope'] = 'user'
      instance.stub(:emails).and_return(emails)
      instance.email.should eq('first@example.com')
    end
  end

  context "#raw_info" do
    it "should use relative paths" do
      instance = subject.new({})
      instance.stub(:access_token).and_return(access_token)
      access_token.should_receive(:get).with('user').and_return(response)
      instance.raw_info.should eq(parsed_response)
    end
  end

  context "#emails" do
    it "should use relative paths" do
      instance = subject.new({})
      instance.stub(:access_token).and_return(access_token)
      access_token.should_receive(:get).with('user/emails', :headers=>{"Accept"=>"application/vnd.outrightmental.v1"}).and_return(response)
      instance.options['scope'] = 'user'
      instance.emails.should eq(parsed_response)
    end
  end

  context '#info.urls' do
    it 'should use html_url from raw_info' do
      instance = subject.new({})
      instance.stub(:raw_info).and_return({ 'login' => 'me', 'html_url' => 'http://enterprise/me' })
      instance.info['urls']['OutrightMental'].should == 'http://enterprise/me'
    end
  end

end
